@@ -36,6 +36,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <ctype.h>
 #include <getopt.h>
 #include <math.h>
 #include <errno.h>
@@ -107,7 +108,7 @@ char *replace_str_all(char *str, char *orig, char *rep) {
   char *p, *p_str = str, *p_buffer = (char *)buffer;
   int len = strlen(str);
   strncpy(p_buffer, str, len);
-  while (p = strstr(p_str, orig)) {
+  while ((p = strstr(p_str, orig))) {
     strncpy(p_buffer, p_str, p-p_str);
     p_buffer += (p-p_str);
     len = len - strlen(orig) + strlen(rep);    
@@ -143,6 +144,9 @@ void json_add_key_value(FILE *output, char *key, void *value, int type, int last
             p = (char *)value;
             p = replace_str_all(p, "\\", "\\\\");
             p = replace_str_all(p, "\"", "\\\"");
+            p = replace_str_all(p, "\n", "");
+            p = replace_str_all(p, "\f", "");
+            p = replace_str_all(p, "\r", "");
             fprintf(output, "\"%s\": \"%s\"", key, p);
             free(p);
             break;
@@ -165,42 +169,17 @@ void json_add_key_value(FILE *output, char *key, void *value, int type, int last
 
 void json_codec_info(FILE *output, AVCodecContext *enc, int indent) {
     const char *codec_name;
-    AVCodec *p;
     char buf1[32];
     int bitrate;
     AVRational display_aspect_ratio;
 
-    p = avcodec_find_decoder(enc->codec_id);
-
-    if (p) {
-        codec_name = p->name;
-    } else if (enc->codec_id == CODEC_ID_MPEG2TS) {
-        /* fake mpeg2 transport stream codec (currently not
-           registered) */
-        codec_name = "mpeg2ts";
-    } else if (enc->codec_name[0] != '\0') {
-        codec_name = enc->codec_name;
-    } else {
-        /* output avi tags */
-        if(   isprint(enc->codec_tag&0xFF) && isprint((enc->codec_tag>>8)&0xFF)
-           && isprint((enc->codec_tag>>16)&0xFF) && isprint((enc->codec_tag>>24)&0xFF)){
-            snprintf(buf1, sizeof(buf1), "%c%c%c%c / 0x%04X",
-                     enc->codec_tag & 0xff,
-                     (enc->codec_tag >> 8) & 0xff,
-                     (enc->codec_tag >> 16) & 0xff,
-                     (enc->codec_tag >> 24) & 0xff,
-                      enc->codec_tag);
-        } else {
-            snprintf(buf1, sizeof(buf1), "0x%04x", enc->codec_tag);
-        }
-        codec_name = buf1;
-    }
+    codec_name  = avcodec_get_name(enc->codec_id);
 
     switch(enc->codec_type) {
     case AVMEDIA_TYPE_VIDEO:
         codec_name = fix_codec_name(codec_name);
         json_add_key_value(output, "codec", (void *)codec_name, JSON_STRING, 0, indent);
-        if (enc->pix_fmt != PIX_FMT_NONE) {
+        if (enc->pix_fmt != AV_PIX_FMT_NONE) {
             json_add_key_value(output, "pixel_format", (void *)av_get_pix_fmt_name(enc->pix_fmt), JSON_STRING, 0, indent);
         }
         if (enc->width) {
@@ -235,37 +214,37 @@ void json_codec_info(FILE *output, AVCodecContext *enc, int indent) {
 
         /* for PCM codecs, compute bitrate directly */
         switch(enc->codec_id) {
-        case CODEC_ID_PCM_F64BE:
-        case CODEC_ID_PCM_F64LE:
+        case AV_CODEC_ID_PCM_F64BE:
+        case AV_CODEC_ID_PCM_F64LE:
             bitrate = enc->sample_rate * enc->channels * 64;
             break;
-        case CODEC_ID_PCM_S32LE:
-        case CODEC_ID_PCM_S32BE:
-        case CODEC_ID_PCM_U32LE:
-        case CODEC_ID_PCM_U32BE:
-        case CODEC_ID_PCM_F32BE:
-        case CODEC_ID_PCM_F32LE:
+        case AV_CODEC_ID_PCM_S32LE:
+        case AV_CODEC_ID_PCM_S32BE:
+        case AV_CODEC_ID_PCM_U32LE:
+        case AV_CODEC_ID_PCM_U32BE:
+        case AV_CODEC_ID_PCM_F32BE:
+        case AV_CODEC_ID_PCM_F32LE:
             bitrate = enc->sample_rate * enc->channels * 32;
             break;
-        case CODEC_ID_PCM_S24LE:
-        case CODEC_ID_PCM_S24BE:
-        case CODEC_ID_PCM_U24LE:
-        case CODEC_ID_PCM_U24BE:
-        case CODEC_ID_PCM_S24DAUD:
+        case AV_CODEC_ID_PCM_S24LE:
+        case AV_CODEC_ID_PCM_S24BE:
+        case AV_CODEC_ID_PCM_U24LE:
+        case AV_CODEC_ID_PCM_U24BE:
+        case AV_CODEC_ID_PCM_S24DAUD:
             bitrate = enc->sample_rate * enc->channels * 24;
             break;
-        case CODEC_ID_PCM_S16LE:
-        case CODEC_ID_PCM_S16BE:
-        case CODEC_ID_PCM_S16LE_PLANAR:
-        case CODEC_ID_PCM_U16LE:
-        case CODEC_ID_PCM_U16BE:
+        case AV_CODEC_ID_PCM_S16LE:
+        case AV_CODEC_ID_PCM_S16BE:
+        case AV_CODEC_ID_PCM_S16LE_PLANAR:
+        case AV_CODEC_ID_PCM_U16LE:
+        case AV_CODEC_ID_PCM_U16BE:
             bitrate = enc->sample_rate * enc->channels * 16;
             break;
-        case CODEC_ID_PCM_S8:
-        case CODEC_ID_PCM_U8:
-        case CODEC_ID_PCM_ALAW:
-        case CODEC_ID_PCM_MULAW:
-        case CODEC_ID_PCM_ZORK:
+        case AV_CODEC_ID_PCM_S8:
+        case AV_CODEC_ID_PCM_U8:
+        case AV_CODEC_ID_PCM_ALAW:
+        case AV_CODEC_ID_PCM_MULAW:
+        case AV_CODEC_ID_PCM_ZORK:
             bitrate = enc->sample_rate * enc->channels * 8;
             break;
         default:
@@ -300,54 +279,6 @@ void json_codec_info(FILE *output, AVCodecContext *enc, int indent) {
     }
 }
 
-
-static void json_stream_format(FILE *output, AVFormatContext *ic, int i, int indent, int first, int type_filter) {
-    static int _first = 1;
-    char buf1[32];
-
-    AVStream *st = ic->streams[i];
-
-    if (first)
-        _first = 1;
-
-    if(st->codec->codec_type == type_filter){
-        if (!_first)
-            fprintf(output, ", ");
-        _first = 0;
-        fprintf(output, "{\n");
-
-        json_codec_info(output, st->codec, indent + 1);
-        if(st->codec->codec_type == AVMEDIA_TYPE_VIDEO){
-            if (st->time_base.den && st->time_base.num && av_q2d(st->time_base) > 0.001) {
-                snprintf(buf1, sizeof(buf1), "%d:%d",
-                         st->time_base.den, st->time_base.num);
-                json_add_key_value(output, "framerate", buf1, JSON_STRING, 0, indent + 1);
-            } else {
-                snprintf(buf1, sizeof(buf1), "%d:%d",
-                         st->r_frame_rate.num, st->r_frame_rate.den);
-                json_add_key_value(output, "framerate", buf1, JSON_STRING, 0, indent + 1);
-            }
-            if (st->sample_aspect_ratio.num && // default
-                av_cmp_q(st->sample_aspect_ratio, st->codec->sample_aspect_ratio)) {
-                AVRational display_aspect_ratio;
-                av_reduce(&display_aspect_ratio.num, &display_aspect_ratio.den,
-                          st->codec->width*st->sample_aspect_ratio.num,
-                          st->codec->height*st->sample_aspect_ratio.den,
-                          1024*1024);
-                snprintf(buf1, sizeof(buf1), "%d:%d",
-                         st->sample_aspect_ratio.num, st->sample_aspect_ratio.den);
-                json_add_key_value(output, "pixel_aspect_ratio", buf1, JSON_STRING, 0, indent+1);
-                snprintf(buf1, sizeof(buf1), "%d:%d",
-                         display_aspect_ratio.num, display_aspect_ratio.den);
-                json_add_key_value(output, "display_aspect_ratio", buf1, JSON_STRING, 0, indent+1);
-            }
-        }
-        json_add_key_value(output, "id", &i, JSON_INT, 1, indent + 1);
-        do_indent(output, indent-1);
-        fprintf(output, "}");
-    }
-}
-
 static int utf8_validate (char *s, int n) {
   int i;
   int extra_bytes;
@@ -387,38 +318,79 @@ static int utf8_validate (char *s, int n) {
 error:
   return i == n;
 }
-int _json_metadata(FILE *output, AVDictionary *m, int first, int indent)
+
+
+void json_metadata(FILE *output, AVDictionary *m, int indent)
 {
-    int i = 0;
+    int first = 1;
     AVDictionaryEntry *tag = NULL;
     while ((tag = av_dict_get(m, "", tag, AV_DICT_IGNORE_SUFFIX))) {
         if (strlen(tag->value) && utf8_validate (tag->value, strlen(tag->value))) {
             if (first) {
                 first = 0;
-                do_indent(output, 1);
+                do_indent(output, indent);
                 fprintf(output, "\"metadata\": {\n");
+                do_indent(output, indent + 1);
             } else {
-                do_indent(output, 2);
+                do_indent(output, indent + 1);
                 fprintf(output, ",");
-                indent=0;
             }
-            json_add_key_value(output, tag->key, tag->value, JSON_STRING, 1, indent);
+            json_add_key_value(output, tag->key, tag->value, JSON_STRING, 1, 0);
         }
     }
-    return first;
+    if (!first) {
+        do_indent(output, indent);
+        fprintf(output, "},\n");
+    }
 }
 
-void json_metadata(FILE *output, const AVFormatContext *av)
-{
-    int first = 1, indent=2, i=0;
-    first = _json_metadata(output, av->metadata, first, indent);
 
-    for(i=0;i<av->nb_streams;i++) {
-        first = _json_metadata(output, av->streams[i]->metadata, first, indent);
-    }
-    if (!first) {
-        do_indent(output, 1);
-        fprintf(output, "},\n");
+
+static void json_stream_format(FILE *output, AVFormatContext *ic, int i, int indent, int first, int type_filter) {
+    static int _first = 1;
+    char buf1[32];
+
+    AVStream *st = ic->streams[i];
+
+    if (first)
+        _first = 1;
+
+    if(st->codec->codec_type == type_filter){
+        if (!_first)
+            fprintf(output, ", ");
+        _first = 0;
+        fprintf(output, "{\n");
+
+        json_codec_info(output, st->codec, indent + 1);
+        if(st->codec->codec_type == AVMEDIA_TYPE_VIDEO){
+            if (st->time_base.den && st->time_base.num && av_q2d(st->time_base) > 0.001) {
+                snprintf(buf1, sizeof(buf1), "%d:%d",
+                         st->time_base.den, st->time_base.num);
+                json_add_key_value(output, "framerate", buf1, JSON_STRING, 0, indent + 1);
+            } else {
+                snprintf(buf1, sizeof(buf1), "%d:%d",
+                         st->avg_frame_rate.num, st->avg_frame_rate.den);
+                json_add_key_value(output, "framerate", buf1, JSON_STRING, 0, indent + 1);
+            }
+            if (st->sample_aspect_ratio.num && // default
+                av_cmp_q(st->sample_aspect_ratio, st->codec->sample_aspect_ratio)) {
+                AVRational display_aspect_ratio;
+                av_reduce(&display_aspect_ratio.num, &display_aspect_ratio.den,
+                          st->codec->width*st->sample_aspect_ratio.num,
+                          st->codec->height*st->sample_aspect_ratio.den,
+                          1024*1024);
+                snprintf(buf1, sizeof(buf1), "%d:%d",
+                         st->sample_aspect_ratio.num, st->sample_aspect_ratio.den);
+                json_add_key_value(output, "pixel_aspect_ratio", buf1, JSON_STRING, 0, indent+1);
+                snprintf(buf1, sizeof(buf1), "%d:%d",
+                         display_aspect_ratio.num, display_aspect_ratio.den);
+                json_add_key_value(output, "display_aspect_ratio", buf1, JSON_STRING, 0, indent+1);
+            }
+        }
+        json_metadata(output, st->metadata, indent + 1);
+        json_add_key_value(output, "id", &i, JSON_INT, 1, indent + 1);
+        do_indent(output, indent-1);
+        fprintf(output, "}");
     }
 }
 
@@ -462,6 +434,8 @@ void json_oshash(FILE *output, char const *filename, int indent) {
     char hash[32];
 #ifdef WIN32
     sprintf(hash,"%016I64x", gen_oshash(filename));
+#elif defined (__SVR4) && defined (__sun)
+    sprintf(hash,"%016llx", gen_oshash(filename));
 #else
     sprintf(hash,"%016qx", gen_oshash(filename));
 #endif
@@ -519,7 +493,7 @@ void json_format_info(FILE* output, AVFormatContext *ic, const char *url) {
             }
         }
         fprintf(output, "],\n");
-        json_metadata(output, ic);
+        json_metadata(output, ic->metadata, 1);
     } else {
         json_add_key_value(output, "code", "badfile", JSON_STRING, 0, 1);
         json_add_key_value(output, "error", "file does not exist or has unknown format.", JSON_STRING, 0, 1);
@@ -536,9 +510,7 @@ void json_format_info(FILE* output, AVFormatContext *ic, const char *url) {
 #ifdef AVINFO
 int main(int argc, char **argv) {
     char inputfile_name[255];
-    AVInputFormat *input_fmt = NULL;
-    AVFormatParameters *formatParams = NULL;
-    AVFormatContext *context;
+    AVFormatContext *context = NULL;
     FILE* output = stdout;
     
     avcodec_register_all();
@@ -553,8 +525,8 @@ int main(int argc, char **argv) {
         output = fopen(argv[2], "w");
     }
     
-    if (av_open_input_file(&context, inputfile_name, input_fmt, 0, formatParams) >= 0) {
-        if (av_find_stream_info(context) >= 0) {
+    if (avformat_open_input(&context, inputfile_name, NULL, NULL) >= 0) {
+        if (avformat_find_stream_info(context, NULL) >= 0) {
             json_format_info(output, context, inputfile_name);
         }
     }
