@@ -2,6 +2,9 @@
 #define _F2T_FFMPEG2THEORA_H_
 
 #include "subtitles.h"
+#include <libavfilter/avfilter.h>
+#include <libavfilter/buffersrc.h>
+#include <libavfilter/buffersink.h>
 
 typedef struct ff2theora_subtitle{
     char *text;
@@ -62,7 +65,6 @@ typedef struct ff2theora{
     double fps;
     struct SwsContext *sws_colorspace_ctx; /* for image resampling/resizing */
     struct SwsContext *sws_scale_ctx; /* for image resampling/resizing */
-    ReSampleContext *audio_resample_ctx;
     ogg_int32_t aspect_numerator;
     ogg_int32_t aspect_denominator;
     int colorspace;
@@ -118,6 +120,13 @@ typedef struct ff2theora{
     unsigned char y_lut[256];
     unsigned char uv_lut[256];
 
+    AVFilterContext *buffersink_ctx;
+    AVFilterContext *buffersrc_ctx;
+    AVFilterGraph *filter_graph;
+    AVFrame *filter_frame;
+    int last_width;
+    int last_height;
+    enum AVPixelFormat last_pixfmt;
 }
 *ff2theora;
 
