@@ -1,11 +1,11 @@
 /* -*- tab-width:4;c-file-style:"cc-mode"; -*- */
 /*
  * subtitles.c -- Kate Subtitles
- * Copyright (C) 2003-2009 <j@v2v.cc>
+ * Copyright (C) 2003-2011 <j@v2v.cc>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
+ * the Free Software Foundation, either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -181,14 +181,17 @@ void report_unknown_subtitle_encoding(const char *name, FILE *frontend)
 
 static char *fgets2(char *s,size_t sz,FILE *f)
 {
-    char *ret = fgets(s, sz, f);
+  char *ret = fgets(s, sz, f);
+  if (ret) {
     /* fixup DOS newline character */
     char *ptr=strchr(s, '\r');
     if (ptr) {
       *ptr='\n';
       *(ptr+1)=0;
     }
-    return ret;
+  }
+  else *s=0;
+  return ret;
 }
 
 static double hmsms2s(int h,int m,int s,int ms)
@@ -331,6 +334,58 @@ static void remove_last_newline(char *text)
   }
 }
 
+static int store_subtitle(ff2theora_kate_stream *this,
+                          char *text, double t0, double t1, int x1, int x2, int y1, int y2,
+                          int ignore_non_utf8,int *warned, unsigned int line, FILE *frontend)
+{
+  char *utf8;
+  size_t len;
+  int ret;
+
+  remove_last_newline(text);
+
+  /* we want all text to be UTF-8 */
+  utf8=convert_subtitle_to_utf8(this->subtitles_encoding,text,ignore_non_utf8, frontend);
+  if (!utf8) {
+    warn(frontend, this->filename, line, "Failed to get UTF-8 text");
+    return -1;
+  }
+
+  len = strlen(utf8);
+  this->subtitles = (ff2theora_subtitle*)realloc(this->subtitles, (this->num_subtitles+1)*sizeof(ff2theora_subtitle));
+  if (!this->subtitles) {
+    free(utf8);
+    warn(frontend, NULL, 0, "Out of memory");
+    return -1;
+  }
+  ret=kate_text_validate(kate_utf8,utf8,len+1);
+  if (ret<0) {
+    if (!*warned) {
+      warn(frontend, this->filename, line, "subtitle is not valid UTF-8: %s",utf8);
+      if (!frontend)
+        fprintf(stderr,"  further invalid subtitles will NOT be flagged\n");
+      *warned=1;
+    }
+  }
+  else {
+    /* kill off trailing \n characters */
+    while (len>0) {
+      if (utf8[len-1]=='\n') utf8[--len]=0; else break;
+    }
+    this->subtitles[this->num_subtitles].text = utf8;
+    this->subtitles[this->num_subtitles].len = len;
+    this->subtitles[this->num_subtitles].t0 = t0;
+    this->subtitles[this->num_subtitles].t1 = t1;
+    this->subtitles[this->num_subtitles].x1 = x1;
+    this->subtitles[this->num_subtitles].x2 = x2;
+    this->subtitles[this->num_subtitles].y1 = y1;
+    this->subtitles[this->num_subtitles].y2 = y2;
+    this->num_subtitles++;
+  }
+
+  return 0;
+}
+
 #endif
 
 int load_subtitles(ff2theora_kate_stream *this, int ignore_non_utf8, FILE *frontend)
@@ -343,14 +398,13 @@ int load_subtitles(ff2theora_kate_stream *this, int ignore_non_utf8, FILE *front
     int id;
     static char text[4096];
     int h0,m0,s0,ms0,h1,m1,s1,ms1;
+    int x1,x2,y1,y2;
     double t0=0.0;
     double t1=0.0;
     static char str[4096];
     int warned=0;
     FILE *f;
-    size_t len;
     unsigned int line=0;
-    char *utf8;
 
     this->subtitles = NULL;
 
@@ -374,7 +428,7 @@ int load_subtitles(ff2theora_kate_stream *this, int ignore_non_utf8, FILE *front
 
     fgets2(str,sizeof(str),f);
     ++line;
-    while (!feof(f)) {
+    while (!feof(f) || *str) {
       switch (need) {
         case need_id:
           if (!strcmp(str,"\n")) {
@@ -397,68 +451,41 @@ int load_subtitles(ff2theora_kate_stream *this, int ignore_non_utf8, FILE *front
           }
           break;
         case need_timing:
-          ret=sscanf(str,"%d:%d:%d%*[.,]%d --> %d:%d:%d%*[.,]%d\n",&h0,&m0,&s0,&ms0,&h1,&m1,&s1,&ms1);
-          if (ret!=8 || (h0|m0|s0|ms0)<0 || (h1|m1|s1|ms1)<0) {
-            warn(frontend, this->filename, line, "Syntax error: %s",str);
+          ret=sscanf(str,"%d:%d:%d%*[.,]%d --> %d:%d:%d%*[.,]%d %*[xX]1: %d %*[xX]2: %d %*[yY]1: %d %*[yY]2: %d\n",&h0,&m0,&s0,&ms0,&h1,&m1,&s1,&ms1,&x1,&x2,&y1,&y2);
+          if (ret!=12) {
+            x1=y1=x2=y2=-INT_MAX;
+            ret=sscanf(str,"%d:%d:%d%*[.,]%d --> %d:%d:%d%*[.,]%d\n",&h0,&m0,&s0,&ms0,&h1,&m1,&s1,&ms1);
+            if (ret!=8) {
+              warn(frontend, this->filename, line, "Syntax error: %s",str);
+              fclose(f);
+              free(this->subtitles);
+              return -1;
+            }
+          }
+          t0=hmsms2s(h0,m0,s0,ms0);
+          t1=hmsms2s(h1,m1,s1,ms1);
+          if ((h0|m0|s0|ms0)<0 || (h1|m1|s1|ms1)<0) {
+            warn(frontend, this->filename, line, "Bad timestamp specification: %s",str);
             fclose(f);
             free(this->subtitles);
             return -1;
           }
-          else {
-            t0=hmsms2s(h0,m0,s0,ms0);
-            t1=hmsms2s(h1,m1,s1,ms1);
-          }
           need=need_text;
           break;
         case need_text:
           if (str[0]=='\n') {
             /* we have all the lines for that subtitle, remove the last \n */
-            remove_last_newline(text);
-
-            /* we want all text to be UTF-8 */
-            utf8=convert_subtitle_to_utf8(this->subtitles_encoding,text,ignore_non_utf8, frontend);
-            if (!utf8) {
-              warn(frontend, this->filename, line, "Failed to get UTF-8 text");
+            int ret = store_subtitle(this, text, t0, t1, x1, x2, y1, y2, ignore_non_utf8, &warned, line, frontend);
+            if (ret < 0) {
               fclose(f);
               free(this->subtitles);
-              return -1;
-              break;
-            }
-
-            len = strlen(utf8);
-            this->subtitles = (ff2theora_subtitle*)realloc(this->subtitles, (this->num_subtitles+1)*sizeof(ff2theora_subtitle));
-            if (!this->subtitles) {
-              free(utf8);
-              warn(frontend, NULL, 0, "Out of memory");
-              fclose(f);
-              free(this->subtitles);
-              return -1;
-            }
-            ret=kate_text_validate(kate_utf8,utf8,len+1);
-            if (ret<0) {
-              if (!warned) {
-                warn(frontend, this->filename, line, "subtitle is not valid UTF-8: %s",utf8);
-                if (!frontend)
-                  fprintf(stderr,"  further invalid subtitles will NOT be flagged\n");
-                warned=1;
-              }
-            }
-            else {
-              /* kill off trailing \n characters */
-              while (len>0) {
-                if (utf8[len-1]=='\n') utf8[--len]=0; else break;
-              }
-              this->subtitles[this->num_subtitles].text = utf8;
-              this->subtitles[this->num_subtitles].len = len;
-              this->subtitles[this->num_subtitles].t0 = t0;
-              this->subtitles[this->num_subtitles].t1 = t1;
-              this->num_subtitles++;
+              return ret;
             }
             need=need_id;
           }
           else {
             /* in case of very long subtitles */
-            len=strlen(text);
+            size_t len=strlen(text);
             if (len+strlen(str) >= sizeof(text)) {
               warn(frontend, this->filename, line, "Subtitle text is too long - truncated");
             }
@@ -473,13 +500,20 @@ int load_subtitles(ff2theora_kate_stream *this, int ignore_non_utf8, FILE *front
 
     fclose(f);
 
-#if 0
-    // there seems to be quite a lot of files like this, so disable this test.
     if (need!=need_id) {
       /* shouldn't be a problem though, but warn */
       warn(frontend, this->filename, line, "Missing data in - truncated file ?");
+
+      /* add any leftover text we've accumulated */
+      if (need == need_text && text[0]) {
+        int ret = store_subtitle(this, text, t0, t1, x1, x2, y1, y2, ignore_non_utf8, &warned, line, frontend);
+        if (ret < 0) {
+          fclose(f);
+          free(this->subtitles);
+          return ret;
+        }
+      }
     }
-#endif
 
     /* fprintf(stderr,"  %u subtitles loaded.\n", this->num_subtitles); */
 
@@ -522,6 +556,10 @@ int add_subtitle_for_stream(ff2theora_kate_stream *streams, int nstreams, int id
         ks->subtitles[ks->num_subtitles].len = utf8len;
         ks->subtitles[ks->num_subtitles].t0 = t;
         ks->subtitles[ks->num_subtitles].t1 = t+duration;
+        ks->subtitles[ks->num_subtitles].x1 = -INT_MAX;
+        ks->subtitles[ks->num_subtitles].x2 = -INT_MAX;
+        ks->subtitles[ks->num_subtitles].y1 = -INT_MAX;
+        ks->subtitles[ks->num_subtitles].y2 = -INT_MAX;
         ks->num_subtitles++;
       }
     }
@@ -530,6 +568,75 @@ int add_subtitle_for_stream(ff2theora_kate_stream *streams, int nstreams, int id
   return 0;
 }
 
+int add_image_subtitle_for_stream(ff2theora_kate_stream *streams, int nstreams, int idx, float t, float duration, const AVSubtitleRect *sr, int org_width, int org_height, FILE *frontend)
+{
+#ifdef HAVE_KATE
+  int n, c, ret;
+  ff2theora_subtitle *subtitle;
+
+  if (sr->nb_colors <= 0 || sr->nb_colors > 256) {
+    warn(frontend, NULL, 0, "Unsupported number of colors in image subtitle: %d", sr->nb_colors);
+    return -1;
+  }
+
+  for (n=0; n<nstreams; ++n) {
+    ff2theora_kate_stream *ks=streams+n;
+    if (idx == ks->stream_index) {
+      ks->subtitles = (ff2theora_subtitle*)realloc(ks->subtitles, (ks->num_subtitles+1)*sizeof(ff2theora_subtitle));
+      if (!ks->subtitles) {
+        warn(frontend, NULL, 0, "Out of memory");
+        return -1;
+      }
+      subtitle = &ks->subtitles[ks->num_subtitles];
+
+      kate_region_init(&subtitle->kr);
+      subtitle->kr.metric = kate_millionths;
+      subtitle->kr.x = 1000000 * sr->x / org_width;
+      subtitle->kr.y = 1000000 * sr->y / org_height;
+      subtitle->kr.w = 1000000 * sr->w / org_width;
+      subtitle->kr.h = 1000000 * sr->h / org_height;
+
+      kate_palette_init(&subtitle->kp);
+      subtitle->kp.ncolors = sr->nb_colors;
+      subtitle->kp.colors = malloc(sizeof(kate_color) * subtitle->kp.ncolors);
+      if (!subtitle->kp.colors) {
+        warn(frontend, NULL, 0, "Out of memory");
+        return -1;
+      }
+      const uint32_t *pal = (const uint32_t*)sr->pict.data[1];
+      for (c=0; c<subtitle->kp.ncolors; ++c) {
+        subtitle->kp.colors[c].a = (pal[c]>>24)&0xff;
+        subtitle->kp.colors[c].r = (pal[c]>>16)&0xff;
+        subtitle->kp.colors[c].g = (pal[c]>>8)&0xff;
+        subtitle->kp.colors[c].b = pal[c]&0xff;
+      }
+
+      kate_bitmap_init(&subtitle->kb);
+      subtitle->kb.type = kate_bitmap_type_paletted;
+      subtitle->kb.width = sr->w;
+      subtitle->kb.height = sr->h;
+      subtitle->kb.bpp = 0;
+      while ((1<<subtitle->kb.bpp) < sr->nb_colors) ++subtitle->kb.bpp;
+      subtitle->kb.pixels = malloc(sr->w*sr->h);
+      if (!subtitle->kb.pixels) {
+        free(subtitle->kp.colors);
+        warn(frontend, NULL, 0, "Out of memory");
+        return -1;
+      }
+
+      /* Not quite sure if the AVPicture line data is supposed to always be packed */
+      memcpy(subtitle->kb.pixels,sr->pict.data[0],sr->w*sr->h);
+
+      subtitle->text = NULL;
+      subtitle->t0 = t;
+      subtitle->t1 = t+duration;
+      ks->num_subtitles++;
+    }
+  }
+#endif
+  return 0;
+}
+
 void free_subtitles(ff2theora this)
 {
     size_t i,n;
